# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - FeedReader Macro

    @copyright: 2008 by Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from MoinMoin.Page import Page
import urllib
import xml.dom.pulldom

Dependencies = ["time"]

MAX_ENTRIES = 5

def text(element):
    nodes = []
    for node in element.childNodes:
        if node.nodeType == node.TEXT_NODE:
            nodes.append(node.nodeValue)
    return "".join(nodes)

def execute(macro, args):
    request = macro.request
    fmt = macro.formatter

    max_entries = MAX_ENTRIES
    args = args.split(",")
    if args:
        try:
            feed_url = args[0]
            max_entries = int(args[1])
        except ValueError:
            pass

    feed = urllib.urlopen(feed_url)
    output = []
    nentries = 0
    try:
        events = xml.dom.pulldom.parse(feed)

        title = None
        link = None
        channel_title_text = None
        channel_link_text = None

        output.append(fmt.bullet_list(on=1))

        in_item = 0
        for event, value in events:
            if event == xml.dom.pulldom.START_ELEMENT:
                if value.nodeName == "item":
                    in_item = 1

                if value.nodeName == "title":
                    events.expandNode(value)
                    title = value
                elif value.nodeName == "link":
                    events.expandNode(value)
                    link = value

                if title is not None and link is not None:
                    link_text = text(link)
                    title_text = text(title)

                    if nentries >= max_entries:
                        title = None
                        link = None
                        continue

                    if in_item:
                        output.append(fmt.listitem(on=1))
                        output.append(fmt.url(on=1, href=link_text))
                        output.append(fmt.icon('www'))
                        output.append(fmt.text(" " + title_text))
                        output.append(fmt.url(on=0))
                        output.append(fmt.listitem(on=0))
                        nentries += 1
                    else:
                        channel_title_text = title_text
                        channel_link_text = link_text

                    title = None
                    link = None

            elif event == xml.dom.pulldom.END_ELEMENT:
                if value.nodeName == "item":
                    in_item = 0

        output.append(fmt.bullet_list(on=0))

        if channel_title_text is not None and channel_link_text is not None:
            output.append(fmt.paragraph(on=1))
            output.append(fmt.url(on=1, href=channel_link_text))
            output.append(fmt.text(channel_title_text))
            output.append(fmt.url(on=0))
            output.append(fmt.text(" "))
            output.append(fmt.url(on=1, href=feed_url))
            output.append(fmt.icon('rss'))
            output.append(fmt.url(on=0))
            output.append(fmt.paragraph(on=0))

    finally:
        feed.close()

    return ''.join(output)

# vim: tabstop=4 expandtab shiftwidth=4
